@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2005 Robert Aman
+# Copyright (c) 2005-2008 Robert Aman
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -21,8 +21,6 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #++
 
-UUID_TOOLS_VERSION = "1.0.2"
-
 $:.unshift(File.dirname(__FILE__))
 
 require 'uri'
@@ -31,6 +29,11 @@ require 'thread'
 require 'digest/sha1'
 require 'digest/md5'
 
+require 'uuidtools/version'
+
+# Backwards compatibility with old method of versioning.
+UUID_TOOLS_VERSION = UUID::UUID_TOOLS_VERSION::STRING
+
 #  Because it's impossible to hype a UUID generator on its genuine merits,
 #  I give you... Really bad ASCII art in the comments:
 #
@@ -417,7 +420,7 @@ class UUID
     for i in 0..5
       result << sprintf("%2.2x", @nodes[i])
     end
-    return result
+    return result.downcase
   end
   
   # Returns an integer representation for this UUID.
@@ -473,9 +476,17 @@ class UUID
   # Returns nil if a MAC address could not be found.
   def self.mac_address #:nodoc:
     if @@mac_address.nil?
-      if RUBY_PLATFORM =~ /solaris/
+      require 'rbconfig'
+      os_platform = Config::CONFIG['target_os']
+      if os_platform =~ /win/ && !(os_platform =~ /darwin/)
+        script_in_path = true
+      else
+        script_in_path = !(`which ifconfig`.strip =~ /no .+ in/)
+      end
+      if os_platform =~ /solaris/
         begin
-        	ifconfig_output = `/sbin/ifconfig -a`
+        	ifconfig_output =
+        	  (script_in_path ? `ifconfig -a` : `/sbin/ifconfig -a`)
         	ip_addresses = ifconfig_output.scan(
         	  /inet\s?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/)
         	ip = ip_addresses.find {|addr| addr[0] != '127.0.0.1'}[0]
@@ -484,14 +495,16 @@ class UUID
         end
         if @@mac_address == "" || @@mac_address == nil
           begin
-  	        ifconfig_output = `ifconfig -a`.split(' ')
+  	        ifconfig_output =
+  	          (script_in_path ?
+  	            `ifconfig -a` : `/sbin/ifconfig -a`).split(' ')
             index = ifconfig_output.index("inet") + 1
             ip = ifconfig_output[index]
             @@mac_address = `arp #{ip}`.split(' ')[3]
           rescue Exception
           end
         end
-      elsif RUBY_PLATFORM =~ /win/ && !(RUBY_PLATFORM =~ /darwin/)
+      elsif os_platform =~ /win/ && !(os_platform =~ /darwin/)
         begin
           ifconfig_output = `ipconfig /all`
           mac_addresses = ifconfig_output.scan(
@@ -504,43 +517,54 @@ class UUID
       else
         begin
           mac_addresses = []
-          if RUBY_PLATFORM =~ /netbsd/
-            ifconfig_output = `/sbin/ifconfig -a 2>&1`
+          if os_platform =~ /netbsd/
+            ifconfig_output =
+              (script_in_path ? `ifconfig -a 2>&1` : `/sbin/ifconfig -a 2>&1`)
             mac_addresses = ifconfig_output.scan(
               Regexp.new("address\: (#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
-          elsif RUBY_PLATFORM =~ /openbsd/
+          elsif os_platform =~ /openbsd/
             ifconfig_output = `/sbin/ifconfig -a 2>&1`
+            ifconfig_output =
+              (script_in_path ? `ifconfig -a 2>&1` : `/sbin/ifconfig -a 2>&1`)
             mac_addresses = ifconfig_output.scan(
               Regexp.new("addr (#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
           elsif File.exists?('/sbin/ifconfig')
-            ifconfig_output = `/sbin/ifconfig 2>&1`
+            ifconfig_output =
+              (script_in_path ? `ifconfig 2>&1` : `/sbin/ifconfig 2>&1`)
             mac_addresses = ifconfig_output.scan(
               Regexp.new("ether (#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
             if mac_addresses.size == 0
-              ifconfig_output = `/sbin/ifconfig -a 2>&1`
+              ifconfig_output =
+                (script_in_path ?
+                  `ifconfig -a 2>&1` : `/sbin/ifconfig -a 2>&1`)
               mac_addresses = ifconfig_output.scan(
                 Regexp.new("ether (#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
             end
             if mac_addresses.size == 0
               ifconfig_output =
-                `/sbin/ifconfig | grep HWaddr | cut -c39- 2>&1`
+                (script_in_path ?
+                  `ifconfig | grep HWaddr | cut -c39- 2>&1` :
+                  `/sbin/ifconfig | grep HWaddr | cut -c39- 2>&1`)
               mac_addresses = ifconfig_output.scan(
                 Regexp.new("(#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
             end
           else
             ifconfig_output =
-              `ifconfig 2>&1`
+              (script_in_path ? `ifconfig 2>&1` : `/sbin/ifconfig 2>&1`)
             mac_addresses = ifconfig_output.scan(
               Regexp.new("ether (#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
             if mac_addresses.size == 0
               ifconfig_output =
-                `ifconfig -a 2>&1`
+                (script_in_path ?
+                  `ifconfig -a 2>&1` : `/sbin/ifconfig -a 2>&1`)
               mac_addresses = ifconfig_output.scan(
                 Regexp.new("ether (#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
             end
             if mac_addresses.size == 0
               ifconfig_output =
-                `ifconfig | grep HWaddr | cut -c39- 2>&1`
+                (script_in_path ?
+                  `ifconfig | grep HWaddr | cut -c39- 2>&1` :
+                  `/sbin/ifconfig | grep HWaddr | cut -c39- 2>&1`)
               mac_addresses = ifconfig_output.scan(
                 Regexp.new("(#{(["[0-9a-fA-F]{2}"] * 6).join(":")})"))
             end
@@ -551,8 +575,16 @@ class UUID
         rescue
         end
       end
-      @@mac_address.downcase! if @@mac_address.respond_to?(:downcase!)
-      @@mac_address.strip! if @@mac_address.respond_to?(:strip!)
+      if @@mac_address != nil
+        if @@mac_address.respond_to?(:to_str)
+          @@mac_address = @@mac_address.to_str
+        else
+          @@mac_address = @@mac_address.to_s
+        end
+        @@mac_address.downcase!
+        @@mac_address.strip!
+      end
+
       # Verify that the MAC address is in the right format.
       # Nil it out if it isn't.
       unless @@mac_address.respond_to?(:scan) &&
